/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Annozilla.
 *
 * The Initial Developer of the Original Code is
 * Matthew Wilson <matthew@mjwilson.demon.co.uk>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * Methods in the "view" interface. An implementation of this
 * interface will be informed of various changes being made to
 * the state of the annotations. Note that a view will usually
 * also implement the nsIAnnotationsObserver interface, and
 * register itself as an observer with the nsIAnnotationsApplication.
 */
function AnnozillaAnnotationsView () {}

/**
 * Called to inform the view of a change in the progress
 * of active operations has changed.
 * @param serversRemaining Number of servers which we are
 * still waiting for.
 * @param target Identifies the set of annotations
 */
AnnozillaAnnotationsView.prototype.updateProgress =
function(serversRemaining, target) {};

/**
 * Called to inform the view that a list of annotations
 * is about to be loaded.
 * @param target Identifies the set of annotations
 */
AnnozillaAnnotationsView.prototype.userLoadBeginning =
function(target) {};

AnnozillaAnnotationsView.prototype.annotationResolutionBeginning =
function (target) {};

AnnozillaAnnotationsView.prototype.annotationResolutionFinished =
function (target) {};

/**
 * Called to inform the view that the annotation
 * is about to be be checked for replies.
 * @param uri URI of the annotation being checked for replies.
 * @param target Identifies the set of annotations
 */
AnnozillaAnnotationsView.prototype.replyCheckBeginning =
function(uri, target) {};

/**
 * Called to inform the view that an annotation
 * is about to be posted.
 * @param target Identifies the set of annotations
 */
AnnozillaAnnotationsView.prototype.postBeginning =
function (target) {};

/**
 * Called to inform the view that an update to an annotation
 * is about to be posted.
 * @param uri URI of the annotation being updated.
 * @param target Identifies the set of annotations
 */
AnnozillaAnnotationsView.prototype.updateBeginning =
function(uri, target) {};

/**
 * Called to inform the view that no replies for an annotation
 * have been found.
 * @param annotation Annotation for which no replies have been found.
 * @param target Identifies the set of annotations
 */
AnnozillaAnnotationsView.prototype.noRepliesFor =
function (annotation, target) {};

/**
 * Called to inform the view that a reply for an annotation
 * has been loaded.Note, the observer interface can also inform the view
 * that a set of replies have been loaded; however, this
 * method is recommended to be used, so that the annotations
 * get resolved and added one-by-one and the "stop load"
 * functionality can be used.
 * @param inReplyTo Annotation which was replied to.
 * @param reply The reply.
 * @param range Range associated with the annotation.
 * @param doc Document associated with the annotation.
 * @param target Identifies the set of annotations
 */
AnnozillaAnnotationsView.prototype.addReply =
function (inReplyTo, reply, range, doc, target) {};

/**
 * Called to inform the view that a tab has been selected.
 * @param browser Selected tab
 */
AnnozillaAnnotationsView.prototype.tabSelected =
function (browser) {};

/**
 * Called to inform the view that the user preference for
 * whether icons should be displayed has changed.
 * @param icons Whether to insert icons
 */
AnnozillaAnnotationsView.prototype.insertIconsPrefChanging =
function (icons) {};

/**
 * Called to inform the view that the user preference for
 * whether highlights should be displayed has changed.
 * @param highlight Whether to insert highlights
 */
AnnozillaAnnotationsView.prototype.insertHighlightPrefChanging =
function (highlight) {};

/**
 * Called to add a single annotation to the view.
 * Note, the observer interface can also inform the view
 * that a set of annotations have been loaded; however, this
 * method is recommended to be used, so that the annotations
 * get resolved and added one-by-one and the "stop load"
 * functionality can be used.
 * @param annot Annotation to add.
 * @param range Range associated with the annotation.
 * @param doc Document associated with the annotation.
 * @param target Identifies the set of annotations
 */
AnnozillaAnnotationsView.prototype.addAnnotation =
function (annot, range, doc, target) {};

/**
 * An ordered list of views.
 */
function AnnozillaViews () {
    this.views = new Array;
};

/**
 * Add a view to the list of the views onto the annotations.
 */
AnnozillaViews.prototype.addView = function (view)
{
    this.views.push (view);
};

/**
 * Remove a view from the list of the views onto the annotations.
 */
AnnozillaViews.prototype.removeView = function (view)
{
    if (this.views.indexOf (view) > -1)
    {
        this.views.splice (this.views.indexOf (view), 1);
    }
};

/**
 * Called to inform all views of a change in the progress
 * of active operations has changed.
 * @param serversRemaining Number of servers which we are
 * still waiting for.
 * @param target Identifies the set of annotations
 */
AnnozillaViews.prototype.updateProgressViews = function (serverCount, target) {
    this.views.forEach (
        function (view, index, views) {
            try {
                view.updateProgress (serverCount, target);
            } catch (e) {
                Components.classes['@mozilla.org/consoleservice;1'].
                   getService(Components.interfaces.nsIConsoleService).
                   logStringMessage ("views.js updateProgressViews " + e);
            }
        }
    );
};

/**
 * Called to inform all views that a list of annotations
 * is about to be loaded.
 * @param target Identifies the set of annotations
 */
AnnozillaViews.prototype.userLoadBeginning =
function(target) {
    this.views.forEach (
        function (view, index, views) {
            try {
                view.userLoadBeginning (target);
            } catch (e) {
                Components.classes['@mozilla.org/consoleservice;1'].
                   getService(Components.interfaces.nsIConsoleService).
                   logStringMessage ("views.js userLoadBeginning " + e);
            }
        }
    );
};

/**
 * Called to inform all views that the annotation
 * is about to be be checked for replies.
 * @param uri URI of the annotation being checked for replies.
 * @param target Identifies the set of annotations
 */
AnnozillaViews.prototype.replyCheckBeginning =
function(uri, target) {
    this.views.forEach (
        function (view, index, views) {
            try {
                view.replyCheckBeginning (uri, target);
            } catch (e) {
                Components.classes['@mozilla.org/consoleservice;1'].
                   getService(Components.interfaces.nsIConsoleService).
                   logStringMessage ("views.js replyCheckBeginning " + e);
            }
        }
    );
};

/**
 * Called to inform all views that the user is creating
 * an annotation to be posted. (NB when the user opens an
 * editor window, not when the data gets posted to the server.)
 */
AnnozillaViews.prototype.postBeginning =
function (target) {
    this.views.forEach (
        function (view, index, views) {
            try {
                view.postBeginning (target);
            } catch (e) {
                Components.classes['@mozilla.org/consoleservice;1'].
                   getService(Components.interfaces.nsIConsoleService).
                   logStringMessage ("views.js postBeginning " + e);
            }
        }
    );
};

/**
 * Called to inform all views that an update to an annotation
 * is about to be posted.
 * @param uri URI of the annotation being updated.
 * @param target Identifies the set of annotations
 */
AnnozillaViews.prototype.updateBeginning =
function(uri, target) {
    this.views.forEach (
        function (view, index, views) {
            try {
                view.updateBeginning (uri, target);
            } catch (e) {
                Components.classes['@mozilla.org/consoleservice;1'].
                   getService(Components.interfaces.nsIConsoleService).
                   logStringMessage ("views.js updateBeginning " + e);
            }
        }
    );
};

/**
 * Called to inform all views that no replies for an annotation
 * have been found.
 * @param annotation Annotation for which no replies have been found.
 * @param target Identifies the set of annotations
 */
AnnozillaViews.prototype.noRepliesFor =
function (annotation, target) {
    this.views.forEach (
        function (view, index, views) {
            try {
                view.noRepliesFor (annotation, target);
            } catch (e) {
                Components.classes['@mozilla.org/consoleservice;1'].
                   getService(Components.interfaces.nsIConsoleService).
                   logStringMessage ("views.js noRepliesFor " + e);
            }
        }
    );
};


/**
 * Called to inform all views that a reply for an annotation
 * has been loaded.Note, the observer interface can also inform the view
 * that a set of replies have been loaded; however, this
 * method is recommended to be used, so that the annotations
 * get resolved and added one-by-one and the "stop load"
 * functionality can be used.
 * @param inReplyTo Annotation which was replied to.
 * @param reply The reply.
 * @param range Range associated with the annotation.
 * @param doc Document associated with the annotation.
 * @param target Identifies the set of annotations
 */
AnnozillaViews.prototype.addReply =
function (inReplyTo, reply, range, doc, target) {
    this.views.forEach (
        function (view, index, views) {
            try {
                view.addReply (inReplyTo, reply, range, doc, target);
            } catch (e) {
                Components.classes['@mozilla.org/consoleservice;1'].
                   getService(Components.interfaces.nsIConsoleService).
                   logStringMessage ("views.js addReply " + e);
                   Components.utils.reportError (e);
            }
        }
    );
};

/**
 * Called to inform all views that a tab has been selected.
 * @param browser Selected tab
 */
AnnozillaViews.prototype.tabSelected =
function (browser) {
    this.views.forEach (
        function (view, index, views) {
            try {
                view.tabSelected (browser);
            } catch (e) {
                Components.classes['@mozilla.org/consoleservice;1'].
                   getService(Components.interfaces.nsIConsoleService).
                   logStringMessage ("views.js tabSelected " + e);
            }
        }
    );
};

/**
 * Called to inform all views that the user preference for
 * whether icons should be displayed has changed.
 * @param insert Whether to insert icons
 */
AnnozillaViews.prototype.insertIconsPrefChanging =
function (insert) {
    this.views.forEach (
        function (view, index, views) {
            try {
                view.insertIconsPrefChanging (insert);
            } catch (e) {
                Components.classes['@mozilla.org/consoleservice;1'].
                   getService(Components.interfaces.nsIConsoleService).
                   logStringMessage ("views.js insertIconsPrefChanging " + e);
            }
        }
    );
};

/**
 * Called to inform all views that the user preference for
 * whether highlights should be displayed has changed.
 * @param insert Whether to insert highlights
 */
AnnozillaViews.prototype.insertHighlightPrefChanging =
function (insert) {
    this.views.forEach (
        function (view, index, views) {
            try {
                view.insertHighlightPrefChanging (insert);
            } catch (e) {
                Components.classes['@mozilla.org/consoleservice;1'].
                   getService(Components.interfaces.nsIConsoleService).
                   logStringMessage ("views.js insertHighlightPrefChanging " + e);
            }
        }
    );
};

/**
 * Called to add a single annotation to all views.
 * Note, the observer interface can also inform the view
 * that a set of annotations have been loaded; however, this
 * method is recommended to be used, so that the annotations
 * get resolved and added one-by-one and the "stop load"
 * functionality can be used.
 * @param annot Annotation to add.
 * @param range Range associated with the annotation.
 * @param doc Document associated with the annotation.
 * @param target Identifies the set of annotations
 */
AnnozillaViews.prototype.addAnnotation =
function (annot, range, doc, target) {
    this.views.forEach (
        function (view, index, views) {
            try {
                view.addAnnotation  (annot, range, doc, target);
            } catch (e) {
                Components.classes['@mozilla.org/consoleservice;1'].
                   getService(Components.interfaces.nsIConsoleService).
                   logStringMessage ("views.js addAnnotation " + e);
                   Components.utils.reportError (e);
            }
        }
    );
};

AnnozillaViews.prototype.annotationResolutionBeginning =
function (target) {
    this.views.forEach (
        function (view, index, views) {
            try {
                view.annotationResolutionBeginning  (target);
            } catch (e) {
                Components.classes['@mozilla.org/consoleservice;1'].
                   getService(Components.interfaces.nsIConsoleService).
                   logStringMessage ("views.js annotationResolutionBeginning " + e);
            }
        }
    );
};

AnnozillaViews.prototype.annotationResolutionFinished =
function (target) {
    this.views.forEach (
        function (view, index, views) {
            try {
                view.annotationResolutionFinished  (target);
            } catch (e) {
                Components.classes['@mozilla.org/consoleservice;1'].
                   getService(Components.interfaces.nsIConsoleService).
                   logStringMessage ("views.js annotationResolutionBeginning " + e);
            }
        }
    );
};